@@ -35,8 +35,8 @@
 #include "py/runtime.h"
 #include "py/stream.h"
 #include "py/mphal.h"
-#include "lib/timeutils/timeutils.h"
-#include "lib/netutils/netutils.h"
+#include "shared/timeutils/timeutils.h"
+#include "shared/netutils/netutils.h"
 #include "modnetwork.h"
 #include "modusocket.h"
 #include "modwlan.h"
