@@ -45,8 +45,7 @@
 #include "py/mphal.h"
 #include "py/stream.h"
 #include "py/mperrno.h"
-#include "lib/netutils/netutils.h"
-#include "tcpip_adapter.h"
+#include "shared/netutils/netutils.h"
 #include "mdns.h"
 #include "modnetwork.h"
 
@@ -56,29 +55,23 @@
 #include "lwip/igmp.h"
 #include "esp_log.h"
 
-#if !MICROPY_ESP_IDF_4
-#define lwip_bind lwip_bind_r
-#define lwip_listen lwip_listen_r
-#define lwip_accept lwip_accept_r
-#define lwip_setsockopt lwip_setsockopt_r
-#define lwip_fnctl lwip_fnctl_r
-#define lwip_recvfrom lwip_recvfrom_r
-#define lwip_write lwip_write_r
-#define lwip_sendto lwip_sendto_r
-#define lwip_close lwip_close_r
-#endif
-
 #define SOCKET_POLL_US (100000)
 #define MDNS_QUERY_TIMEOUT_MS (5000)
 #define MDNS_LOCAL_SUFFIX ".local"
 
+enum {
+    SOCKET_STATE_NEW,
+    SOCKET_STATE_CONNECTED,
+    SOCKET_STATE_PEER_CLOSED,
+};
+
 typedef struct _socket_obj_t {
     mp_obj_base_t base;
     int fd;
     uint8_t domain;
     uint8_t type;
     uint8_t proto;
-    bool peer_closed;
+    uint8_t state;
     unsigned int retries;
     #if MICROPY_PY_USOCKET_EVENTS
     mp_obj_t events_callback;
@@ -153,16 +146,6 @@ void usocket_events_handler(void) {
 
 #endif // MICROPY_PY_USOCKET_EVENTS
 
-NORETURN static void exception_from_errno(int _errno) {
-    // Here we need to convert from lwip errno values to MicroPython's standard ones
-    if (_errno == EADDRINUSE) {
-        _errno = MP_EADDRINUSE;
-    } else if (_errno == EINPROGRESS) {
-        _errno = MP_EINPROGRESS;
-    }
-    mp_raise_OSError(_errno);
-}
-
 static inline void check_for_exceptions(void) {
     mp_handle_pending(true);
 }
@@ -181,7 +164,12 @@ static int _socket_getaddrinfo3(const char *nodename, const char *servname,
         memcpy(nodename_no_local, nodename, nodename_len - local_len);
         nodename_no_local[nodename_len - local_len] = '\0';
 
+        #if ESP_IDF_VERSION < ESP_IDF_VERSION_VAL(4, 1, 0)
         struct ip4_addr addr = {0};
+        #else
+        esp_ip4_addr_t addr = {0};
+        #endif
+
         esp_err_t err = mdns_query_a(nodename_no_local, MDNS_QUERY_TIMEOUT_MS, &addr);
         if (err != ESP_OK) {
             if (err == ESP_ERR_NOT_FOUND) {
@@ -272,7 +260,6 @@ STATIC mp_obj_t socket_make_new(const mp_obj_type_t *type_in, size_t n_args, siz
     sock->domain = AF_INET;
     sock->type = SOCK_STREAM;
     sock->proto = 0;
-    sock->peer_closed = false;
     if (n_args > 0) {
         sock->domain = mp_obj_get_int(args[0]);
         if (n_args > 1) {
@@ -283,9 +270,11 @@ STATIC mp_obj_t socket_make_new(const mp_obj_type_t *type_in, size_t n_args, siz
         }
     }
 
+    sock->state = sock->type == SOCK_STREAM ? SOCKET_STATE_NEW : SOCKET_STATE_CONNECTED;
+
     sock->fd = lwip_socket(sock->domain, sock->type, sock->proto);
     if (sock->fd < 0) {
-        exception_from_errno(errno);
+        mp_raise_OSError(errno);
     }
     _socket_settimeout(sock, UINT64_MAX);
 
@@ -296,10 +285,11 @@ STATIC mp_obj_t socket_bind(const mp_obj_t arg0, const mp_obj_t arg1) {
     socket_obj_t *self = MP_OBJ_TO_PTR(arg0);
     struct addrinfo *res;
     _socket_getaddrinfo(arg1, &res);
+    self->state = SOCKET_STATE_CONNECTED;
     int r = lwip_bind(self->fd, res->ai_addr, res->ai_addrlen);
     lwip_freeaddrinfo(res);
     if (r < 0) {
-        exception_from_errno(errno);
+        mp_raise_OSError(errno);
     }
     return mp_const_none;
 }
@@ -308,9 +298,10 @@ STATIC MP_DEFINE_CONST_FUN_OBJ_2(socket_bind_obj, socket_bind);
 STATIC mp_obj_t socket_listen(const mp_obj_t arg0, const mp_obj_t arg1) {
     socket_obj_t *self = MP_OBJ_TO_PTR(arg0);
     int backlog = mp_obj_get_int(arg1);
+    self->state = SOCKET_STATE_CONNECTED;
     int r = lwip_listen(self->fd, backlog);
     if (r < 0) {
-        exception_from_errno(errno);
+        mp_raise_OSError(errno);
     }
     return mp_const_none;
 }
@@ -331,7 +322,7 @@ STATIC mp_obj_t socket_accept(const mp_obj_t arg0) {
             break;
         }
         if (errno != EAGAIN) {
-            exception_from_errno(errno);
+            mp_raise_OSError(errno);
         }
         check_for_exceptions();
     }
@@ -350,7 +341,7 @@ STATIC mp_obj_t socket_accept(const mp_obj_t arg0) {
     sock->domain = self->domain;
     sock->type = self->type;
     sock->proto = self->proto;
-    sock->peer_closed = false;
+    sock->state = SOCKET_STATE_CONNECTED;
     _socket_settimeout(sock, UINT64_MAX);
 
     // make the return value
@@ -369,11 +360,12 @@ STATIC mp_obj_t socket_connect(const mp_obj_t arg0, const mp_obj_t arg1) {
     struct addrinfo *res;
     _socket_getaddrinfo(arg1, &res);
     MP_THREAD_GIL_EXIT();
+    self->state = SOCKET_STATE_CONNECTED;
     int r = lwip_connect(self->fd, res->ai_addr, res->ai_addrlen);
     MP_THREAD_GIL_ENTER();
     lwip_freeaddrinfo(res);
     if (r != 0) {
-        exception_from_errno(errno);
+        mp_raise_OSError(errno);
     }
 
     return mp_const_none;
@@ -392,7 +384,7 @@ STATIC mp_obj_t socket_setsockopt(size_t n_args, const mp_obj_t *args) {
             int val = mp_obj_get_int(args[3]);
             int ret = lwip_setsockopt(self->fd, SOL_SOCKET, opt, &val, sizeof(int));
             if (ret != 0) {
-                exception_from_errno(errno);
+                mp_raise_OSError(errno);
             }
             break;
         }
@@ -489,11 +481,17 @@ STATIC mp_uint_t _socket_read_data(mp_obj_t self_in, void *buf, size_t size,
     struct sockaddr *from, socklen_t *from_len, int *errcode) {
     socket_obj_t *sock = MP_OBJ_TO_PTR(self_in);
 
+    // A new socket cannot be read from.
+    if (sock->state == SOCKET_STATE_NEW) {
+        *errcode = MP_ENOTCONN;
+        return MP_STREAM_ERROR;
+    }
+
     // If the peer closed the connection then the lwIP socket API will only return "0" once
     // from lwip_recvfrom and then block on subsequent calls.  To emulate POSIX behaviour,
     // which continues to return "0" for each call on a closed socket, we set a flag when
     // the peer closed the socket.
-    if (sock->peer_closed) {
+    if (sock->state == SOCKET_STATE_PEER_CLOSED) {
         return 0;
     }
 
@@ -518,7 +516,7 @@ STATIC mp_uint_t _socket_read_data(mp_obj_t self_in, void *buf, size_t size,
             MP_THREAD_GIL_ENTER();
         }
         if (r == 0) {
-            sock->peer_closed = true;
+            sock->state = SOCKET_STATE_PEER_CLOSED;
         }
         if (r >= 0) {
             return r;
@@ -543,7 +541,7 @@ mp_obj_t _socket_recvfrom(mp_obj_t self_in, mp_obj_t len_in,
     int errcode;
     mp_uint_t ret = _socket_read_data(self_in, vstr.buf, len, from, from_len, &errcode);
     if (ret == MP_STREAM_ERROR) {
-        exception_from_errno(errcode);
+        mp_raise_OSError(errcode);
     }
 
     vstr.len = ret;
@@ -576,8 +574,9 @@ int _socket_send(socket_obj_t *sock, const char *data, size_t datalen) {
         MP_THREAD_GIL_EXIT();
         int r = lwip_write(sock->fd, data + sentlen, datalen - sentlen);
         MP_THREAD_GIL_ENTER();
-        if (r < 0 && errno != EWOULDBLOCK) {
-            exception_from_errno(errno);
+        // lwip returns EINPROGRESS when trying to send right after a non-blocking connect
+        if (r < 0 && errno != EWOULDBLOCK && errno != EINPROGRESS) {
+            mp_raise_OSError(errno);
         }
         if (r > 0) {
             sentlen += r;
@@ -585,7 +584,7 @@ int _socket_send(socket_obj_t *sock, const char *data, size_t datalen) {
         check_for_exceptions();
     }
     if (sentlen == 0) {
-        mp_raise_OSError(MP_ETIMEDOUT);
+        mp_raise_OSError(sock->retries == 0 ? MP_EWOULDBLOCK : MP_ETIMEDOUT);
     }
     return sentlen;
 }
@@ -635,7 +634,7 @@ STATIC mp_obj_t socket_sendto(mp_obj_t self_in, mp_obj_t data_in, mp_obj_t addr_
             return mp_obj_new_int_from_uint(ret);
         }
         if (ret == -1 && errno != EWOULDBLOCK) {
-            exception_from_errno(errno);
+            mp_raise_OSError(errno);
         }
         check_for_exceptions();
     }
@@ -668,7 +667,8 @@ STATIC mp_uint_t socket_stream_write(mp_obj_t self_in, const void *buf, mp_uint_
         if (r > 0) {
             return r;
         }
-        if (r < 0 && errno != EWOULDBLOCK) {
+        // lwip returns MP_EINPROGRESS when trying to write right after a non-blocking connect
+        if (r < 0 && errno != EWOULDBLOCK && errno != EINPROGRESS) {
             *errcode = errno;
             return MP_STREAM_ERROR;
         }
@@ -718,6 +718,12 @@ STATIC mp_uint_t socket_stream_ioctl(mp_obj_t self_in, mp_uint_t request, uintpt
         if (FD_ISSET(socket->fd, &efds)) {
             ret |= MP_STREAM_POLL_HUP;
         }
+
+        // New (unconnected) sockets are writable and have HUP set.
+        if (socket->state == SOCKET_STATE_NEW) {
+            ret |= (arg & MP_STREAM_POLL_WR) | MP_STREAM_POLL_HUP;
+        }
+
         return ret;
     } else if (request == MP_STREAM_CLOSE) {
         if (socket->fd >= 0) {
