@@ -30,13 +30,12 @@ def fs_corrupted():
     while 1:
         print(
             """\
-The filesystem starting at sector %d with size %d sectors appears to
-be corrupted. If you had important data there, you may want to make a flash
-snapshot to try to recover it. Otherwise, perform factory reprogramming
-of MicroPython firmware (completely erase flash, followed by firmware
-programming).
+The filesystem starting at sector %d with size %d sectors looks corrupt.
+You may want to make a flash snapshot and try to recover it. Otherwise,
+format it with uos.VfsLfs2.mkfs(bdev), or completely erase the flash and
+reprogram MicroPython.
 """
-            % (bdev.START_SEC, bdev.blocks)
+            % (bdev.start_sec, bdev.blocks)
         )
         time.sleep(3)
 
