@@ -50,9 +50,27 @@
 #define RCC_SR_RMVF     RCC_CSR_RMVF
 #endif
 
+// Whether this MCU has an independent PLL which can generate 48MHz for USB.
+#if defined(STM32F413xx)
+// STM32F413 uses PLLI2S as secondary PLL.
+#define HAVE_PLL48 1
+#define RCC_CR_PLL48_ON RCC_CR_PLLI2SON
+#define RCC_CR_PLL48_RDY RCC_CR_PLLI2SRDY
+#elif defined(STM32F7)
+// STM32F7 uses PLLSAI as secondary PLL.
+#define HAVE_PLL48 1
+#define RCC_CR_PLL48_ON RCC_CR_PLLSAION
+#define RCC_CR_PLL48_RDY RCC_CR_PLLSAIRDY
+#else
+// MCU does not have a secondary PLL.
+#define HAVE_PLL48 0
+#endif
+
+#if MICROPY_HW_ENTER_BOOTLOADER_VIA_RESET
 // Location in RAM of bootloader state (just after the top of the stack)
 extern uint32_t _estack[];
 #define BL_STATE ((uint32_t *)&_estack)
+#endif
 
 static inline void powerctrl_disable_hsi_if_unused(void) {
     #if !MICROPY_HW_CLK_USE_HSI && (defined(STM32F4) || defined(STM32F7) || defined(STM32H7))
@@ -62,32 +80,47 @@ static inline void powerctrl_disable_hsi_if_unused(void) {
 }
 
 NORETURN void powerctrl_mcu_reset(void) {
+    #if MICROPY_HW_ENTER_BOOTLOADER_VIA_RESET
     BL_STATE[1] = 1; // invalidate bootloader address
     #if __DCACHE_PRESENT == 1
     SCB_CleanDCache();
     #endif
-    NVIC_SystemReset();
-}
-
-NORETURN void powerctrl_enter_bootloader(uint32_t r0, uint32_t bl_addr) {
-    BL_STATE[0] = r0;
-    BL_STATE[1] = bl_addr;
-    #if __DCACHE_PRESENT == 1
-    SCB_CleanDCache();
     #endif
     NVIC_SystemReset();
 }
 
-static __attribute__((naked)) void branch_to_bootloader(uint32_t r0, uint32_t bl_addr) {
+NORETURN static __attribute__((naked)) void branch_to_bootloader(uint32_t r0, uint32_t bl_addr) {
     __asm volatile (
         "ldr r2, [r1, #0]\n"    // get address of stack pointer
         "msr msp, r2\n"         // get stack pointer
         "ldr r2, [r1, #4]\n"    // get address of destination
         "bx r2\n"               // branch to bootloader
         );
+    MP_UNREACHABLE;
+}
+
+NORETURN void powerctrl_enter_bootloader(uint32_t r0, uint32_t bl_addr) {
+    #if MICROPY_HW_ENTER_BOOTLOADER_VIA_RESET
+
+    // Enter the bootloader via a reset, so everything is reset (including WDT).
+    // Upon reset powerctrl_check_enter_bootloader() will jump to the bootloader.
+    BL_STATE[0] = r0;
+    BL_STATE[1] = bl_addr;
+    #if __DCACHE_PRESENT == 1
+    SCB_CleanDCache();
+    #endif
+    NVIC_SystemReset();
+
+    #else
+
+    // Enter the bootloader via a direct jump.
+    branch_to_bootloader(r0, bl_addr);
+
+    #endif
 }
 
 void powerctrl_check_enter_bootloader(void) {
+    #if MICROPY_HW_ENTER_BOOTLOADER_VIA_RESET
     uint32_t bl_addr = BL_STATE[1];
     BL_STATE[1] = 1; // invalidate bootloader address
     if ((bl_addr & 0xfff) == 0 && (RCC->RCC_SR & RCC_SR_SFTRSTF)) {
@@ -99,6 +132,7 @@ void powerctrl_check_enter_bootloader(void) {
         uint32_t r0 = BL_STATE[0];
         branch_to_bootloader(r0, bl_addr);
     }
+    #endif
 }
 
 #if !defined(STM32F0) && !defined(STM32L0) && !defined(STM32WB)
@@ -141,13 +175,24 @@ STATIC int powerctrl_config_vos(uint32_t sysclk_mhz) {
 }
 
 // Assumes that PLL is used as the SYSCLK source
-int powerctrl_rcc_clock_config_pll(RCC_ClkInitTypeDef *rcc_init, uint32_t sysclk_mhz, bool need_pllsai) {
+int powerctrl_rcc_clock_config_pll(RCC_ClkInitTypeDef *rcc_init, uint32_t sysclk_mhz, bool need_pll48) {
     uint32_t flash_latency;
 
-    #if defined(STM32F7)
-    if (need_pllsai) {
-        // Configure PLLSAI at 48MHz for those peripherals that need this freq
-        // (calculation assumes it can get an integral value of PLLSAIN)
+    #if HAVE_PLL48
+    if (need_pll48) {
+        // Configure secondary PLL at 48MHz for those peripherals that need this freq
+        // (the calculation assumes it can get an integral value of PLL-N).
+
+        #if defined(STM32F413xx)
+        const uint32_t plli2sm = HSE_VALUE / 1000000;
+        const uint32_t plli2sq = 2;
+        const uint32_t plli2sr = 2;
+        const uint32_t plli2sn = 48 * plli2sq;
+        RCC->PLLI2SCFGR = plli2sr << RCC_PLLI2SCFGR_PLLI2SR_Pos
+            | plli2sq << RCC_PLLI2SCFGR_PLLI2SQ_Pos
+            | plli2sn << RCC_PLLI2SCFGR_PLLI2SN_Pos
+            | plli2sm << RCC_PLLI2SCFGR_PLLI2SM_Pos;
+        #else
         const uint32_t pllm = (RCC->PLLCFGR >> RCC_PLLCFGR_PLLM_Pos) & 0x3f;
         const uint32_t pllsaip = 4;
         const uint32_t pllsaiq = 2;
@@ -155,13 +200,18 @@ int powerctrl_rcc_clock_config_pll(RCC_ClkInitTypeDef *rcc_init, uint32_t sysclk
         RCC->PLLSAICFGR = pllsaiq << RCC_PLLSAICFGR_PLLSAIQ_Pos
             | (pllsaip / 2 - 1) << RCC_PLLSAICFGR_PLLSAIP_Pos
             | pllsain << RCC_PLLSAICFGR_PLLSAIN_Pos;
-        RCC->CR |= RCC_CR_PLLSAION;
+        #endif
+
+        // Turn on the PLL and wait for it to be ready.
+        RCC->CR |= RCC_CR_PLL48_ON;
         uint32_t ticks = mp_hal_ticks_ms();
-        while (!(RCC->CR & RCC_CR_PLLSAIRDY)) {
+        while (!(RCC->CR & RCC_CR_PLL48_RDY)) {
             if (mp_hal_ticks_ms() - ticks > 200) {
                 return -MP_ETIMEDOUT;
             }
         }
+
+        // Select the alternate 48MHz source.
         RCC->DCKCFGR2 |= RCC_DCKCFGR2_CK48MSEL;
     }
     #endif
@@ -317,7 +367,7 @@ int powerctrl_set_sysclk(uint32_t sysclk, uint32_t ahb, uint32_t apb1, uint32_t
     // Default PLL parameters that give 48MHz on PLL48CK
     uint32_t m = MICROPY_HW_CLK_VALUE / 1000000, n = 336, p = 2, q = 7;
     uint32_t sysclk_source;
-    bool need_pllsai = false;
+    bool need_pll48 = false;
 
     // Search for a valid PLL configuration that keeps USB at 48MHz
     uint32_t sysclk_mhz = sysclk / 1000000;
@@ -338,8 +388,8 @@ int powerctrl_set_sysclk(uint32_t sysclk, uint32_t ahb, uint32_t apb1, uint32_t
                 uint32_t vco_out = sys * p;
                 n = vco_out * m / (MICROPY_HW_CLK_VALUE / 1000000);
                 q = vco_out / 48;
-                #if defined(STM32F7)
-                need_pllsai = vco_out % 48 != 0;
+                #if HAVE_PLL48
+                need_pll48 = vco_out % 48 != 0;
                 #endif
             }
             goto set_clk;
@@ -377,8 +427,8 @@ set_clk:
     RCC_ClkInitStruct.APB2CLKDivider = calc_apb2_div(ahb / apb2);
     #if defined(STM32H7)
     RCC_ClkInitStruct.SYSCLKDivider = RCC_SYSCLK_DIV1;
-    RCC_ClkInitStruct.APB3CLKDivider = RCC_APB3_DIV2;
-    RCC_ClkInitStruct.APB4CLKDivider = RCC_APB4_DIV2;
+    RCC_ClkInitStruct.APB3CLKDivider = MICROPY_HW_CLK_APB3_DIV;
+    RCC_ClkInitStruct.APB4CLKDivider = MICROPY_HW_CLK_APB4_DIV;
     #endif
 
     #if MICROPY_HW_CLK_LAST_FREQ
@@ -393,11 +443,11 @@ set_clk:
         return -MP_EIO;
     }
 
-    #if defined(STM32F7)
+    #if HAVE_PLL48
     // Deselect PLLSAI as 48MHz source if we were using it
     RCC->DCKCFGR2 &= ~RCC_DCKCFGR2_CK48MSEL;
     // Turn PLLSAI off because we are changing PLLM (which drives PLLSAI)
-    RCC->CR &= ~RCC_CR_PLLSAION;
+    RCC->CR &= ~RCC_CR_PLL48_ON;
     #endif
 
     // Re-configure PLL
@@ -440,7 +490,7 @@ set_clk:
     // Set PLL as system clock source if wanted
     if (sysclk_source == RCC_SYSCLKSOURCE_PLLCLK) {
         RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_SYSCLK;
-        int ret = powerctrl_rcc_clock_config_pll(&RCC_ClkInitStruct, sysclk_mhz, need_pllsai);
+        int ret = powerctrl_rcc_clock_config_pll(&RCC_ClkInitStruct, sysclk_mhz, need_pll48);
         if (ret != 0) {
             return ret;
         }
@@ -469,10 +519,103 @@ set_clk:
 
 #elif defined(STM32WB)
 
+#include "stm32wbxx_ll_utils.h"
+
+#define LPR_THRESHOLD (2000000)
+#define VOS2_THRESHOLD (16000000)
+
+enum {
+    SYSCLK_MODE_NONE,
+    SYSCLK_MODE_MSI,
+    SYSCLK_MODE_HSE_64M,
+};
+
 int powerctrl_set_sysclk(uint32_t sysclk, uint32_t ahb, uint32_t apb1, uint32_t apb2) {
-    // For now it's not supported to change SYSCLK (only bus dividers).
-    if (sysclk != HAL_RCC_GetSysClockFreq()) {
-        return -MP_EINVAL;
+    int sysclk_mode = SYSCLK_MODE_NONE;
+    uint32_t msirange = 0;
+    uint32_t sysclk_cur = HAL_RCC_GetSysClockFreq();
+
+    if (sysclk == sysclk_cur) {
+        // SYSCLK does not need changing.
+    } else if (sysclk == 64000000) {
+        sysclk_mode = SYSCLK_MODE_HSE_64M;
+    } else {
+        for (msirange = 0; msirange < MP_ARRAY_SIZE(MSIRangeTable); ++msirange) {
+            if (MSIRangeTable[msirange] != 0 && sysclk == MSIRangeTable[msirange]) {
+                sysclk_mode = SYSCLK_MODE_MSI;
+                break;
+            }
+        }
+
+        if (sysclk_mode == SYSCLK_MODE_NONE) {
+            // Unsupported SYSCLK value.
+            return -MP_EINVAL;
+        }
+    }
+
+    // Exit LPR if SYSCLK will increase beyond threshold.
+    if (LL_PWR_IsEnabledLowPowerRunMode()) {
+        if (sysclk > LPR_THRESHOLD) {
+            if (sysclk_cur < LPR_THRESHOLD) {
+                // Must select MSI=LPR_THRESHOLD=2MHz to exit LPR.
+                LL_RCC_MSI_SetRange(LL_RCC_MSIRANGE_5);
+            }
+
+            // Exit LPR and wait for the regulator to be ready.
+            LL_PWR_ExitLowPowerRunMode();
+            while (!LL_PWR_IsActiveFlag_REGLPF()) {
+            }
+        }
+    }
+
+    // Select VOS1 if SYSCLK will increase beyond threshold.
+    if (sysclk > VOS2_THRESHOLD) {
+        LL_PWR_SetRegulVoltageScaling(LL_PWR_REGU_VOLTAGE_SCALE1);
+        while (LL_PWR_IsActiveFlag_VOS()) {
+        }
+    }
+
+    if (sysclk_mode == SYSCLK_MODE_HSE_64M) {
+        SystemClock_Config();
+    } else if (sysclk_mode == SYSCLK_MODE_MSI) {
+        // Set flash latency to maximum to ensure the latency is large enough for
+        // both the current SYSCLK and the SYSCLK that will be selected below.
+        LL_FLASH_SetLatency(LL_FLASH_LATENCY_3);
+        while (LL_FLASH_GetLatency() != LL_FLASH_LATENCY_3) {
+        }
+
+        // Before changing the MSIRANGE value, if MSI is on then it must also be ready.
+        while ((RCC->CR & (RCC_CR_MSIRDY | RCC_CR_MSION)) == RCC_CR_MSION) {
+        }
+        LL_RCC_MSI_SetRange(msirange << RCC_CR_MSIRANGE_Pos);
+
+        // Clock SYSCLK from MSI.
+        LL_RCC_SetSysClkSource(LL_RCC_SYS_CLKSOURCE_MSI);
+        while (LL_RCC_GetSysClkSource() != LL_RCC_SYS_CLKSOURCE_STATUS_MSI) {
+        }
+
+        // Disable PLL to decrease power consumption.
+        LL_RCC_PLL_Disable();
+        while (LL_RCC_PLL_IsReady() != 0) {
+        }
+        LL_RCC_PLL_DisableDomain_SYS();
+
+        // Select VOS2 if possible.
+        if (sysclk <= VOS2_THRESHOLD) {
+            LL_PWR_SetRegulVoltageScaling(LL_PWR_REGU_VOLTAGE_SCALE2);
+        }
+
+        // Enter LPR if possible.
+        if (sysclk <= LPR_THRESHOLD) {
+            LL_PWR_EnterLowPowerRunMode();
+        }
+
+        // Configure flash latency for the new SYSCLK.
+        LL_SetFlashLatency(sysclk);
+
+        // Update HAL state and SysTick.
+        SystemCoreClockUpdate();
+        powerctrl_config_systick();
     }
 
     // Return straightaway if the clocks are already at the desired frequency.
@@ -607,11 +750,11 @@ void powerctrl_enter_stop_mode(void) {
 
     powerctrl_disable_hsi_if_unused();
 
-    #if defined(STM32F7)
+    #if HAVE_PLL48
     if (RCC->DCKCFGR2 & RCC_DCKCFGR2_CK48MSEL) {
         // Enable PLLSAI if it is selected as 48MHz source
-        RCC->CR |= RCC_CR_PLLSAION;
-        while (!(RCC->CR & RCC_CR_PLLSAIRDY)) {
+        RCC->CR |= RCC_CR_PLL48_ON;
+        while (!(RCC->CR & RCC_CR_PLL48_RDY)) {
         }
     }
     #endif
@@ -712,10 +855,14 @@ void powerctrl_enter_standby_mode(void) {
     RTC->ISR &= ~ISR_BITS;
 
     #if defined(STM32F7)
+    // Save EWUP state
+    uint32_t csr2_ewup = PWR->CSR2 & (PWR_CSR2_EWUP6 | PWR_CSR2_EWUP5 | PWR_CSR2_EWUP4 | PWR_CSR2_EWUP3 | PWR_CSR2_EWUP2 | PWR_CSR2_EWUP1);
     // disable wake-up flags
     PWR->CSR2 &= ~(PWR_CSR2_EWUP6 | PWR_CSR2_EWUP5 | PWR_CSR2_EWUP4 | PWR_CSR2_EWUP3 | PWR_CSR2_EWUP2 | PWR_CSR2_EWUP1);
     // clear global wake-up flag
     PWR->CR2 |= PWR_CR2_CWUPF6 | PWR_CR2_CWUPF5 | PWR_CR2_CWUPF4 | PWR_CR2_CWUPF3 | PWR_CR2_CWUPF2 | PWR_CR2_CWUPF1;
+    // Restore EWUP state
+    PWR->CSR2 |= csr2_ewup;
     #elif defined(STM32H7)
     EXTI_D1->PR1 = 0x3fffff;
     PWR->WKUPCR |= PWR_WAKEUP_FLAG1 | PWR_WAKEUP_FLAG2 | PWR_WAKEUP_FLAG3 | PWR_WAKEUP_FLAG4 | PWR_WAKEUP_FLAG5 | PWR_WAKEUP_FLAG6;
