@@ -32,8 +32,8 @@
 #include "py/stream.h"
 #include "py/mperrno.h"
 #include "py/mphal.h"
-#include "lib/utils/interrupt_char.h"
-#include "lib/utils/mpirq.h"
+#include "shared/runtime/interrupt_char.h"
+#include "shared/runtime/mpirq.h"
 #include "uart.h"
 #include "irq.h"
 #include "pendsv.h"
@@ -118,16 +118,10 @@ STATIC const pyb_uart_irq_map_t mp_uart_irq_map[] = {
 void uart_init0(void) {
     #if defined(STM32H7)
     RCC_PeriphCLKInitTypeDef RCC_PeriphClkInit = {0};
-    // Configure USART1/6 clock source
-    RCC_PeriphClkInit.PeriphClockSelection = RCC_PERIPHCLK_USART16;
+    // Configure USART1/6 and USART2/3/4/5/7/8 clock sources
+    RCC_PeriphClkInit.PeriphClockSelection = RCC_PERIPHCLK_USART16 | RCC_PERIPHCLK_USART234578;
     RCC_PeriphClkInit.Usart16ClockSelection = RCC_USART16CLKSOURCE_D2PCLK2;
-    if (HAL_RCCEx_PeriphCLKConfig(&RCC_PeriphClkInit) != HAL_OK) {
-        __fatal_error("HAL_RCCEx_PeriphCLKConfig");
-    }
-
-    // Configure USART2/3/4/5/7/8 clock source
-    RCC_PeriphClkInit.PeriphClockSelection = RCC_PERIPHCLK_USART234578;
-    RCC_PeriphClkInit.Usart16ClockSelection = RCC_USART234578CLKSOURCE_D2PCLK1;
+    RCC_PeriphClkInit.Usart234578ClockSelection = RCC_USART234578CLKSOURCE_D2PCLK1;
     if (HAL_RCCEx_PeriphCLKConfig(&RCC_PeriphClkInit) != HAL_OK) {
         __fatal_error("HAL_RCCEx_PeriphCLKConfig");
     }
@@ -201,6 +195,11 @@ bool uart_exists(int uart_id) {
             return true;
         #endif
 
+        #if defined(MICROPY_HW_LPUART1_TX) && defined(MICROPY_HW_LPUART1_RX)
+        case PYB_LPUART_1:
+            return true;
+        #endif
+
         default:
             return false;
     }
@@ -211,6 +210,7 @@ bool uart_init(pyb_uart_obj_t *uart_obj,
     uint32_t baudrate, uint32_t bits, uint32_t parity, uint32_t stop, uint32_t flow) {
     USART_TypeDef *UARTx;
     IRQn_Type irqn;
+    uint8_t uart_fn = AF_FN_UART;
     int uart_unit;
 
     const pin_obj_t *pins[4] = {0};
@@ -223,6 +223,16 @@ bool uart_init(pyb_uart_obj_t *uart_obj,
             irqn = USART1_IRQn;
             pins[0] = MICROPY_HW_UART1_TX;
             pins[1] = MICROPY_HW_UART1_RX;
+            #if defined(MICROPY_HW_UART1_RTS)
+            if (flow & UART_HWCONTROL_RTS) {
+                pins[2] = MICROPY_HW_UART1_RTS;
+            }
+            #endif
+            #if defined(MICROPY_HW_UART1_CTS)
+            if (flow & UART_HWCONTROL_CTS) {
+                pins[3] = MICROPY_HW_UART1_CTS;
+            }
+            #endif
             __HAL_RCC_USART1_CLK_ENABLE();
             break;
         #endif
@@ -322,6 +332,16 @@ bool uart_init(pyb_uart_obj_t *uart_obj,
             #endif
             pins[0] = MICROPY_HW_UART5_TX;
             pins[1] = MICROPY_HW_UART5_RX;
+            #if defined(MICROPY_HW_UART5_RTS)
+            if (flow & UART_HWCONTROL_RTS) {
+                pins[2] = MICROPY_HW_UART5_RTS;
+            }
+            #endif
+            #if defined(MICROPY_HW_UART5_CTS)
+            if (flow & UART_HWCONTROL_CTS) {
+                pins[3] = MICROPY_HW_UART5_CTS;
+            }
+            #endif
             break;
         #endif
 
@@ -364,6 +384,16 @@ bool uart_init(pyb_uart_obj_t *uart_obj,
             #endif
             pins[0] = MICROPY_HW_UART7_TX;
             pins[1] = MICROPY_HW_UART7_RX;
+            #if defined(MICROPY_HW_UART7_RTS)
+            if (flow & UART_HWCONTROL_RTS) {
+                pins[2] = MICROPY_HW_UART7_RTS;
+            }
+            #endif
+            #if defined(MICROPY_HW_UART7_CTS)
+            if (flow & UART_HWCONTROL_CTS) {
+                pins[3] = MICROPY_HW_UART7_CTS;
+            }
+            #endif
             break;
         #endif
 
@@ -381,6 +411,16 @@ bool uart_init(pyb_uart_obj_t *uart_obj,
             #endif
             pins[0] = MICROPY_HW_UART8_TX;
             pins[1] = MICROPY_HW_UART8_RX;
+            #if defined(MICROPY_HW_UART8_RTS)
+            if (flow & UART_HWCONTROL_RTS) {
+                pins[2] = MICROPY_HW_UART8_RTS;
+            }
+            #endif
+            #if defined(MICROPY_HW_UART8_CTS)
+            if (flow & UART_HWCONTROL_CTS) {
+                pins[3] = MICROPY_HW_UART8_CTS;
+            }
+            #endif
             break;
         #endif
 
@@ -406,17 +446,40 @@ bool uart_init(pyb_uart_obj_t *uart_obj,
             break;
         #endif
 
+        #if defined(MICROPY_HW_LPUART1_TX) && defined(MICROPY_HW_LPUART1_RX)
+        case PYB_LPUART_1:
+            uart_fn = AF_FN_LPUART;
+            uart_unit = 1;
+            UARTx = LPUART1;
+            irqn = LPUART1_IRQn;
+            pins[0] = MICROPY_HW_LPUART1_TX;
+            pins[1] = MICROPY_HW_LPUART1_RX;
+            #if defined(MICROPY_HW_LPUART1_RTS)
+            if (flow & UART_HWCONTROL_RTS) {
+                pins[2] = MICROPY_HW_LPUART1_RTS;
+            }
+            #endif
+            #if defined(MICROPY_HW_LPUART1_CTS)
+            if (flow & UART_HWCONTROL_CTS) {
+                pins[3] = MICROPY_HW_LPUART1_CTS;
+            }
+            #endif
+            __HAL_RCC_LPUART1_CLK_ENABLE();
+            break;
+        #endif
+
         default:
             // UART does not exist or is not configured for this board
             return false;
     }
 
     uint32_t mode = MP_HAL_PIN_MODE_ALT;
-    uint32_t pull = MP_HAL_PIN_PULL_UP;
 
     for (uint i = 0; i < 4; i++) {
         if (pins[i] != NULL) {
-            bool ret = mp_hal_pin_config_alt(pins[i], mode, pull, AF_FN_UART, uart_unit);
+            // Configure pull-up on RX and CTS (the input pins).
+            uint32_t pull = (i & 1) ? MP_HAL_PIN_PULL_UP : MP_HAL_PIN_PULL_NONE;
+            bool ret = mp_hal_pin_config_alt(pins[i], mode, pull, uart_fn, uart_unit);
             if (!ret) {
                 return false;
             }
@@ -425,7 +488,7 @@ bool uart_init(pyb_uart_obj_t *uart_obj,
 
     uart_obj->uartx = UARTx;
 
-    // init UARTx
+    // Set the initialisation parameters for the UART.
     UART_HandleTypeDef huart;
     memset(&huart, 0, sizeof(huart));
     huart.Instance = UARTx;
@@ -436,6 +499,26 @@ bool uart_init(pyb_uart_obj_t *uart_obj,
     huart.Init.Mode = UART_MODE_TX_RX;
     huart.Init.HwFlowCtl = flow;
     huart.Init.OverSampling = UART_OVERSAMPLING_16;
+    #if !defined(STM32F4)
+    huart.Init.OneBitSampling = UART_ONE_BIT_SAMPLE_DISABLE;
+    #endif
+
+    #if defined(STM32H7) || defined(STM32WB)
+    // Compute the smallest prescaler that will allow the given baudrate.
+    uint32_t presc = UART_PRESCALER_DIV1;
+    if (uart_obj->uart_id == PYB_LPUART_1) {
+        uint32_t source_clk = uart_get_source_freq(uart_obj);
+        for (; presc < UART_PRESCALER_DIV256; ++presc) {
+            uint32_t brr = UART_DIV_LPUART(source_clk, baudrate, presc);
+            if (brr <= LPUART_BRR_MASK) {
+                break;
+            }
+        }
+    }
+    huart.Init.ClockPrescaler = presc;
+    #endif
+
+    // Initialise the UART hardware.
     HAL_UART_Init(&huart);
 
     // Disable all individual UART IRQs, but enable the global handler
@@ -596,6 +679,13 @@ void uart_deinit(pyb_uart_obj_t *self) {
         __HAL_RCC_UART10_RELEASE_RESET();
         __HAL_RCC_UART10_CLK_DISABLE();
     #endif
+    #if defined(LPUART1)
+    } else if (self->uart_id == PYB_LPUART_1) {
+        HAL_NVIC_DisableIRQ(LPUART1_IRQn);
+        __HAL_RCC_LPUART1_FORCE_RESET();
+        __HAL_RCC_LPUART1_RELEASE_RESET();
+        __HAL_RCC_LPUART1_CLK_DISABLE();
+    #endif
     }
 }
 
@@ -603,7 +693,7 @@ void uart_attach_to_repl(pyb_uart_obj_t *self, bool attached) {
     self->attached_to_repl = attached;
 }
 
-uint32_t uart_get_baudrate(pyb_uart_obj_t *self) {
+uint32_t uart_get_source_freq(pyb_uart_obj_t *self) {
     uint32_t uart_clk = 0;
 
     #if defined(STM32F0)
@@ -672,10 +762,42 @@ uint32_t uart_get_baudrate(pyb_uart_obj_t *self) {
     }
     #endif
 
-    // This formula assumes UART_OVERSAMPLING_16
-    uint32_t baudrate = uart_clk / self->uartx->BRR;
+    return uart_clk;
+}
 
-    return baudrate;
+uint32_t uart_get_baudrate(pyb_uart_obj_t *self) {
+    #if defined(LPUART1)
+    if (self->uart_id == PYB_LPUART_1) {
+        return LL_LPUART_GetBaudRate(self->uartx, uart_get_source_freq(self)
+            #if defined(STM32H7) || defined(STM32WB)
+            , self->uartx->PRESC
+            #endif
+            );
+    }
+    #endif
+    return LL_USART_GetBaudRate(self->uartx, uart_get_source_freq(self),
+        #if defined(STM32H7) || defined(STM32WB)
+        self->uartx->PRESC,
+        #endif
+        LL_USART_OVERSAMPLING_16);
+}
+
+void uart_set_baudrate(pyb_uart_obj_t *self, uint32_t baudrate) {
+    #if defined(LPUART1)
+    if (self->uart_id == PYB_LPUART_1) {
+        LL_LPUART_SetBaudRate(self->uartx, uart_get_source_freq(self),
+            #if defined(STM32H7) || defined(STM32WB)
+            LL_LPUART_PRESCALER_DIV1,
+            #endif
+            baudrate);
+        return;
+    }
+    #endif
+    LL_USART_SetBaudRate(self->uartx, uart_get_source_freq(self),
+        #if defined(STM32H7) || defined(STM32WB)
+        LL_USART_PRESCALER_DIV1,
+        #endif
+        LL_USART_OVERSAMPLING_16, baudrate);
 }
 
 mp_uint_t uart_rx_any(pyb_uart_obj_t *self) {
