@@ -1,5 +1,8 @@
 #include <zephyr.h>
-#include "lib/utils/interrupt_char.h"
+#include "shared/runtime/interrupt_char.h"
+
+void mp_hal_init(void);
+void mp_hal_wait_sem(struct k_sem *sem, uint32_t timeout_ms);
 
 static inline mp_uint_t mp_hal_ticks_us(void) {
     return k_cyc_to_ns_floor64(k_cycle_get_32()) / 1000;
@@ -21,7 +24,7 @@ static inline void mp_hal_delay_us(mp_uint_t delay) {
 }
 
 static inline void mp_hal_delay_ms(mp_uint_t delay) {
-    k_msleep(delay);
+    mp_hal_wait_sem(NULL, delay);
 }
 
 static inline uint64_t mp_hal_time_ns(void) {
