@@ -1,11 +1,16 @@
 # test subclassing framebuf.FrameBuffer
 
 try:
-    import framebuf
+    import framebuf, usys
 except ImportError:
     print("SKIP")
     raise SystemExit
 
+# This test and its .exp file is based on a little-endian architecture.
+if usys.byteorder != "little":
+    print("SKIP")
+    raise SystemExit
+
 
 class FB(framebuf.FrameBuffer):
     def __init__(self, n):
