@@ -13,8 +13,13 @@ def thread_entry():
     _thread.exit()
 
 
-_thread.start_new_thread(thread_entry, ())
-_thread.start_new_thread(thread_entry, ())
+for i in range(2):
+    while True:
+        try:
+            _thread.start_new_thread(thread_entry, ())
+            break
+        except OSError:
+            pass
 
 # wait for threads to finish
 time.sleep(1)
