@@ -41,7 +41,12 @@ n_finished = 0
 # set stack size and spawn a few threads
 _thread.stack_size(sz)
 for i in range(n_thread):
-    _thread.start_new_thread(thread_entry, ())
+    while True:
+        try:
+            _thread.start_new_thread(thread_entry, ())
+            break
+        except OSError:
+            pass
 
 # reset stack size to default (for subsequent scripts on baremetal)
 _thread.stack_size()
